package me.axieum.mcmod.authme.impl.gui;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.http.conn.ConnectTimeoutException;
import me.axieum.mcmod.authme.api.gui.AuthScreen;
import me.axieum.mcmod.authme.api.util.MicrosoftUtils;
import me.axieum.mcmod.authme.api.util.SessionUtils;
import net.minecraft.class_124;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_370;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_4587;

import static me.axieum.mcmod.authme.impl.AuthMe.LOGGER;

/**
 * A screen for handling user authentication via Microsoft.
 */
public class MicrosoftAuthScreen extends AuthScreen
{
    // The executor to run the login task on
    private ExecutorService executor = null;
    // The completable future for all Microsoft login tasks
    private CompletableFuture<Void> task = null;
    // The current progress/status of the login task
    private class_2561 status = null;

    /**
     * Constructs a new authentication via Microsoft screen.
     *
     * @param parentScreen  parent (or last) screen that opened this screen
     * @param successScreen screen to be returned to after a successful login
     */
    public MicrosoftAuthScreen(class_437 parentScreen, class_437 successScreen)
    {
        super(new class_2588("gui.authme.microsoft.title"), parentScreen, successScreen);
        this.closeOnSuccess = true;
    }

    @Override
    protected void method_25426()
    {
        super.method_25426();
        assert field_22787 != null;

        // Add a cancel button to abort the task
        final class_4185 cancelBtn;
        method_25411(
            cancelBtn = new class_4185(
                field_22789 / 2 - 50, field_22790 / 2 + 22, 100, 20,
                new class_2588("gui.cancel"),
                button -> method_25419()
            )
        );

        // Set the initial progress/status of the login task
        status = new class_2588("gui.authme.microsoft.status.checkBrowser");

        // Prepare a new executor thread to run the login task on
        executor = Executors.newSingleThreadExecutor();

        // Start the login task
        task = MicrosoftUtils
            // Acquire a Microsoft auth code
            .acquireMSAuthCode(success -> new class_2588("gui.authme.microsoft.browser").getString(), executor)

            // Exchange the Microsoft auth code for an access token
            .thenComposeAsync(msAuthCode -> {
                status = new class_2588("gui.authme.microsoft.status.msAccessToken");
                return MicrosoftUtils.acquireMSAccessToken(msAuthCode, executor);
            })

            // Exchange the Microsoft access token for an Xbox access token
            .thenComposeAsync(msAccessToken -> {
                status = new class_2588("gui.authme.microsoft.status.xboxAccessToken");
                return MicrosoftUtils.acquireXboxAccessToken(msAccessToken, executor);
            })

            // Exchange the Xbox access token for an XSTS token
            .thenComposeAsync(xboxAccessToken -> {
                status = new class_2588("gui.authme.microsoft.status.xboxXstsToken");
                return MicrosoftUtils.acquireXboxXstsToken(xboxAccessToken, executor);
            })

            // Exchange the Xbox XSTS token for a Minecraft access token
            .thenComposeAsync(xboxXstsData -> {
                status = new class_2588("gui.authme.microsoft.status.mcAccessToken");
                return MicrosoftUtils.acquireMCAccessToken(
                    xboxXstsData.get("Token"), xboxXstsData.get("uhs"), executor
                );
            })

            // Build a new Minecraft session with the Minecraft access token
            .thenComposeAsync(mcToken -> {
                status = new class_2588("gui.authme.microsoft.status.mcProfile");
                return MicrosoftUtils.login(mcToken, executor);
            })

            // Update the game session and greet the player
            .thenAccept(session -> {
                // Apply the new session
                SessionUtils.setSession(session);
                // Add a toast that greets the player
                class_370.method_27024(
                    field_22787.method_1566(), class_370.class_371.field_2218,
                    new class_2588("gui.authme.toast.greeting", new class_2585(session.method_1676())), null
                );
                // Mark the task as successful, in turn closing the screen
                LOGGER.info("Successfully logged in via Microsoft!");
                success = true;
            })

            // On any exception, update the status and cancel button
            .exceptionally(error -> {
                status = new class_2588(
                    error.getCause() instanceof ConnectTimeoutException ? "gui.authme.error.timeout"
                                                                        : "gui.authme.error.generic"
                ).method_27692(class_124.field_1061);
                cancelBtn.method_25355(new class_2588("gui.back"));
                return null; // return a default value
            });
    }

    @Override
    public void method_25394(class_4587 matrices, int mouseX, int mouseY, float delta)
    {
        assert field_22787 != null;

        // Render the background before any widgets
        method_25420(matrices);

        // Render a title for the screen
        method_27534(matrices, field_22787.field_1772, field_22785, field_22789 / 2, field_22790 / 2 - 32, 0xffffff);

        // Render the current progress/status of the login, if present
        if (status != null) {
            method_27534(matrices, field_22787.field_1772, status, field_22789 / 2, field_22790 / 2 - 6, 0xdddddd);
        }

        // Cascade the rendering
        super.method_25394(matrices, mouseX, mouseY, delta);
    }

    @Override
    public void method_25419()
    {
        // Cancel the login task if still running
        if (task != null && !task.isDone()) {
            task.cancel(true);
            executor.shutdownNow();
        }

        // Cascade the closing
        super.method_25419();
    }
}
