package me.axieum.mcmod.authme.impl.gui;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.mojang.authlib.exceptions.InvalidCredentialsException;
import me.axieum.mcmod.authme.api.gui.AuthScreen;
import me.axieum.mcmod.authme.api.gui.widget.PasswordFieldWidget;
import me.axieum.mcmod.authme.api.util.MojangUtils;
import me.axieum.mcmod.authme.api.util.SessionUtils;
import net.minecraft.class_124;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_342;
import net.minecraft.class_370;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_4587;

import static me.axieum.mcmod.authme.impl.AuthMe.CONFIG;
import static me.axieum.mcmod.authme.impl.AuthMe.LOGGER;
import static me.axieum.mcmod.authme.impl.AuthMe.getConfig;

/**
 * A screen for handling user authentication via Mojang (or legacy).
 */
public class MojangAuthScreen extends AuthScreen
{
    // The executor to run the login task on
    private ExecutorService executor = null;
    // The completable future for all Mojang login tasks
    private CompletableFuture<Void> task = null;
    // The current progress/status of the login task
    private class_2561 status = null;
    // The username and password text field widgets
    private class_342 usernameField, passwordField;
    // The login button widget
    private class_4185 loginBtn;

    /**
     * Constructs a new authentication via Mojang (or legacy) screen.
     *
     * @param parentScreen  parent (or last) screen that opened this screen
     * @param successScreen screen to be returned to after a successful login
     */
    public MojangAuthScreen(class_437 parentScreen, class_437 successScreen)
    {
        super(new class_2588("gui.authme.mojang.title"), parentScreen, successScreen);
        this.enableKeyboardRepeatEvents = true;
        this.closeOnSuccess = true;
    }

    @Override
    protected void method_25426()
    {
        super.method_25426();
        assert field_22787 != null;

        // Add a username text field
        method_25429(
            usernameField = new class_342(
                field_22787.field_1772,
                field_22789 / 2 - 100, field_22790 / 2 - 39, 200, 20,
                new class_2588("gui.authme.mojang.field.username")
            )
        );
        usernameField.method_1880(128);
        if (getConfig().methods.mojang.lastUsername != null) {
            usernameField.method_1852(getConfig().methods.mojang.lastUsername);
        }
        usernameField.method_1863(value -> loginBtn.field_22763 = isFormValid());

        // Add a password text field
        method_25429(
            passwordField = new PasswordFieldWidget(
                field_22787.field_1772,
                field_22789 / 2 - 100, field_22790 / 2 + 6, 200, 20,
                new class_2588("gui.authme.mojang.field.password")
            )
        );
        passwordField.method_1863(value -> loginBtn.field_22763 = isFormValid());

        // Add a login button to submit the form
        method_25411(
            loginBtn = new class_4185(
                field_22789 / 2 - 100 - 2, field_22790 / 2 + 59, 100, 20,
                new class_2588("gui.authme.mojang.button.login"),
                button -> login()
            )
        );
        loginBtn.field_22763 = isFormValid();

        // Add a cancel button to abort the task
        method_25411(
            new class_4185(
                field_22789 / 2 + 2, field_22790 / 2 + 59, 100, 20,
                new class_2588("gui.cancel"),
                button -> method_25419()
            )
        );
    }

    /**
     * Attempts to log into Mojang with the provided credentials.
     */
    public void login()
    {
        assert field_22787 != null;

        // Check whether the form is valid
        if (!isFormValid()) return;

        // Disable the form fields while logging in
        usernameField.field_22763 = false;
        passwordField.field_22763 = false;
        loginBtn.field_22763 = false;

        // Set the initial progress/status of the login task
        status = new class_2588("gui.authme.mojang.status.loggingIn");

        // Prepare a new executor thread to run the login task on
        executor = Executors.newSingleThreadExecutor();

        // Start the login task
        task = MojangUtils
            // Log into Mojang with username and password and hence build a new session
            .login(usernameField.method_1882(), passwordField.method_1882(), executor)

            // Update the game session, sync the config and greet the player
            .thenAccept(session -> {
                // Apply the new session
                SessionUtils.setSession(session);
                // Sync configuration with the last used username
                getConfig().methods.mojang.lastUsername = usernameField.method_1882();
                CONFIG.save();
                // Add a toast that greets the player
                class_370.method_27024(
                    field_22787.method_1566(), class_370.class_371.field_2218,
                    new class_2588("gui.authme.toast.greeting", new class_2585(session.method_1676())), null
                );
                // Mark the task as successful, in turn closing the screen
                LOGGER.info("Successfully logged in via Mojang (or legacy)!");
                success = true;
            })

            // On any exception, update the status and re-enable form fields
            .exceptionally(error -> {
                status = new class_2588(
                    error.getCause() instanceof InvalidCredentialsException ? "gui.authme.error.credentials"
                                                                            : "gui.authme.error.generic"
                ).method_27692(class_124.field_1061);
                usernameField.field_22763 = true;
                passwordField.field_22763 = true;
                return null; // return a default value
            });
    }

    /**
     * Checks whether the form can be submitted, and hence logged in.
     *
     * @return true if both username & password fields are valid
     */
    public boolean isFormValid()
    {
        return !usernameField.method_1882().isEmpty() && !passwordField.method_1882().isEmpty();
    }

    @Override
    public void method_25394(class_4587 matrices, int mouseX, int mouseY, float delta)
    {
        assert field_22787 != null;

        // Render the background before any widgets
        method_25420(matrices);

        // Render a title for the screen
        method_27534(matrices, field_22787.field_1772, field_22785, field_22789 / 2, usernameField.field_22761 - 16 - 23, 0xffffff);

        // Render the username & password field labels
        method_27535(
            matrices, field_22787.field_1772, usernameField.method_25369(), usernameField.field_22760, usernameField.field_22761 - 16, 0xa0a0a0
        );
        method_27535(
            matrices, field_22787.field_1772, passwordField.method_25369(), passwordField.field_22760, passwordField.field_22761 - 16, 0xa0a0a0
        );

        // Render the current progress/status of the login, if present
        if (status != null) {
            method_27534(matrices, field_22787.field_1772, status, field_22789 / 2, loginBtn.field_22761 - 20, 0xdddddd);
        }

        // Render any text field widgets
        usernameField.method_25394(matrices, mouseX, mouseY, delta);
        passwordField.method_25394(matrices, mouseX, mouseY, delta);

        // Cascade the rendering
        super.method_25394(matrices, mouseX, mouseY, delta);
    }

    @Override
    public boolean method_25422()
    {
        return !usernameField.method_25370() && !passwordField.method_25370();
    }

    @Override
    public void method_25419()
    {
        // Cancel the login task if still running
        if (task != null && !task.isDone()) {
            task.cancel(true);
            executor.shutdownNow();
        }

        // Cascade the closing
        super.method_25419();
    }
}
