package me.axieum.mcmod.authme.impl.gui;

import me.axieum.mcmod.authme.api.gui.AuthScreen;
import me.axieum.mcmod.authme.api.util.SessionUtils;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_342;
import net.minecraft.class_370;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_4587;

import static me.axieum.mcmod.authme.impl.AuthMe.CONFIG;
import static me.axieum.mcmod.authme.impl.AuthMe.LOGGER;
import static me.axieum.mcmod.authme.impl.AuthMe.getConfig;

/**
 * A screen for handling offline user authentication.
 */
public class OfflineAuthScreen extends AuthScreen
{
    // The username and password text field widgets
    private class_342 usernameField;
    // The login button widget
    private class_4185 loginBtn;

    /**
     * Constructs a new offline authentication screen.
     *
     * @param parentScreen  parent (or last) screen that opened this screen
     * @param successScreen screen to be returned to after a successful login
     */
    public OfflineAuthScreen(class_437 parentScreen, class_437 successScreen)
    {
        super(new class_2588("gui.authme.offline.title"), parentScreen, successScreen);
        this.enableKeyboardRepeatEvents = true;
        this.closeOnSuccess = true;
    }

    @Override
    protected void method_25426()
    {
        super.method_25426();
        assert field_22787 != null;

        // Add a username text field
        method_25429(
            usernameField = new class_342(
                field_22787.field_1772,
                field_22789 / 2 - 100, field_22790 / 2 - 6, 200, 20,
                new class_2588("gui.authme.offline.field.username")
            )
        );
        usernameField.method_1880(128);
        if (getConfig().methods.offline.lastUsername != null) {
            usernameField.method_1852(getConfig().methods.offline.lastUsername);
        }
        usernameField.method_1863(value -> loginBtn.field_22763 = isFormValid());

        // Add a login button to submit the form
        method_25411(
            loginBtn = new class_4185(
                field_22789 / 2 - 100 - 2, field_22790 / 2 + 26, 100, 20,
                new class_2588("gui.authme.offline.button.login"),
                button -> login()
            )
        );
        loginBtn.field_22763 = isFormValid();

        // Add a cancel button to abort the task
        method_25411(
            new class_4185(
                field_22789 / 2 + 2, field_22790 / 2 + 26, 100, 20,
                new class_2588("gui.cancel"),
                button -> method_25419()
            )
        );
    }

    /**
     * Creates a new offline-mode session the provided username.
     */
    public void login()
    {
        assert field_22787 != null;

        // Check whether the form is valid
        if (!isFormValid()) return;

        // Disable the form fields while logging in
        usernameField.field_22763 = false;
        loginBtn.field_22763 = false;

        // Create and apply a new offline Minecraft session
        SessionUtils.setSession(SessionUtils.offline(usernameField.method_1882()));

        // Sync configuration with the last used username
        getConfig().methods.offline.lastUsername = usernameField.method_1882();
        CONFIG.save();

        // Add a toast that greets the player
        class_370.method_27024(
            field_22787.method_1566(), class_370.class_371.field_2218,
            new class_2588("gui.authme.toast.greeting", new class_2585(usernameField.method_1882())), null
        );

        // Mark the task as successful, in turn closing the screen
        LOGGER.info("Successfully logged in offline-mode!");
        success = true;
    }

    /**
     * Checks whether the form can be submitted, and hence logged in.
     *
     * @return true if the username field is valid
     */
    public boolean isFormValid()
    {
        return !usernameField.method_1882().isEmpty();
    }

    @Override
    public void method_25394(class_4587 matrices, int mouseX, int mouseY, float delta)
    {
        assert field_22787 != null;

        // Render the background before any widgets
        method_25420(matrices);

        // Render a title for the screen
        method_27534(matrices, field_22787.field_1772, field_22785, field_22789 / 2, usernameField.field_22761 - 16 - 23, 0xffffff);

        // Render the username field label
        method_27535(
            matrices, field_22787.field_1772, usernameField.method_25369(), usernameField.field_22760, usernameField.field_22761 - 16, 0xa0a0a0
        );

        // Render any text field widgets
        usernameField.method_25394(matrices, mouseX, mouseY, delta);

        // Cascade the rendering
        super.method_25394(matrices, mouseX, mouseY, delta);
    }
}
