package me.axieum.mcmod.authme.mixin;

import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import me.axieum.mcmod.authme.impl.gui.AuthMethodScreen;
import net.minecraft.class_2561;
import net.minecraft.class_2588;
import net.minecraft.class_4185;
import net.minecraft.class_419;
import net.minecraft.class_437;

import static me.axieum.mcmod.authme.impl.AuthMe.LOGGER;

/**
 * Injects a button into the disconnected screen to open the authentication screen.
 */
@Mixin(class_419.class)
public abstract class DisconnectedScreenMixin extends class_437
{
    @Shadow
    @Final
    private class_437 parent;

    @Shadow
    @Final
    private class_2561 reason;

    private DisconnectedScreenMixin(class_2561 title)
    {
        super(title);
    }

    /**
     * Injects into the creation of the screen and adds the authentication button.
     *
     * @param ci injection callback info
     */
    @Inject(method = "init", at = @At("TAIL"))
    private void init(CallbackInfo ci)
    {
        // Determine if the disconnection reason is user or session related
        if (isUserRelated(reason)) {
            LOGGER.info("Adding auth button to the disconnected screen");
            assert field_22787 != null;

            // Create and add the button to the screen where the back button is
            final class_4185 backButton = (class_4185) method_25396().get(0);
            method_25411(
                new class_4185(
                    backButton.field_22760,
                    backButton.field_22761,
                    backButton.method_25368(),
                    backButton.method_25364(),
                    new class_2588("gui.authme.button.relogin"),
                    btn -> field_22787.method_1507(new AuthMethodScreen(parent))
                )
            );

            // Shift the back button down below our new button
            backButton.field_22761 += backButton.method_25364() + 4;
        }
    }

    /**
     * Determines if a server disconnection reason is user or session related.
     *
     * @param reason disconnect reason text
     * @return true if the disconnection reason is user or session related
     */
    private static boolean isUserRelated(final @Nullable class_2561 reason)
    {
        if (reason instanceof class_2588) {
            final String key = ((class_2588) reason).method_11022();
            return key != null && (key.startsWith("disconnect.loginFailed")
                || "multiplayer.disconnect.duplicate_login".equals(key)
                || "multiplayer.disconnect.unverified_username".equals(key)
                || "multiplayer.disconnect.not_whitelisted".equals(key)
                || "multiplayer.disconnect.name_taken".equals(key));
        }
        return false;
    }
}
